﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;

namespace AjaxControlToolkit.Reference.Core.Rendering {

    public class HtmlDocRenderer {

        public string RenderDescription(string summary) {
            var sb = new StringBuilder();
            RenderAutoGeneratedComment(sb);
            sb.Append(HttpUtility.HtmlEncode(summary));
            return sb.ToString();
        }

        public string RenderMembers(TypeDoc doc) {
            var sb = new StringBuilder();
            RenderAutoGeneratedComment(sb);
            RenderList(sb, doc.Properties.OrderBy(p => p.Name), "Properties");

            if(doc.Methods.Any()) {
                RenderNewLine(sb);
                RenderList(sb, doc.Methods.OrderBy(m => m.Name), "Methods", ExtenderDoc.BuildMethodSignature);
            }

            if(doc.Events.Any()) {
                RenderNewLine(sb);
                RenderList(sb, doc.Events.OrderBy(e => e.Name), "Events");
            }

            return sb.ToString();
        }

        private void RenderAutoGeneratedComment(StringBuilder sb) {
            RenderComment(sb, "This file is generated automatically. Do not change its content.");
        }

        void RenderComment(StringBuilder sb, string text) {
            sb.Append("<!-- ");
            sb.Append(text);
            sb.AppendLine(" -->");
        }

        void RenderNewLine(StringBuilder sb) {
            sb.Append("<br />");
        }

        void RenderList(StringBuilder sb, IEnumerable<DocBase> members, string header, Func<DocBase, string> memberNameTransform = null) {
            sb.AppendLine(RenderBold(header));
            sb.AppendLine("<ul>");

            foreach(var member in members)
                RenderListItem(sb, member, memberNameTransform);

            sb.AppendLine("</ul>");            
        }

        void RenderListItem(StringBuilder sb, DocBase member, Func<DocBase, string> memberNameTransform = null) {
            sb.Append("\t");
            sb.Append("<li>");
            var memberName = memberNameTransform == null ? member.Name : memberNameTransform(member);
            sb.Append(RenderBold(memberName));
            sb.Append(" - ");
            sb.Append(member.Summary);
            sb.AppendLine("</li>");
        }

        string RenderBold(string text) {
            return String.Format("<b>{0}</b>", text);
        }
    }
}